/*
 * Project name:
     SerialRAM
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20100429:
       - initial release;
 * Description:
     Many embedded systems needs some amount of volatile storage for temporary
     data. Serial SRAMs are popular choice for volatile storage because of their
     small footprint, low I/O pin requirement, low-power consumption and low cost.
     This example writes and reads data from 23K640 Serial RAM device in byte
     read/write operation mode using standard SPI communication. For more
     information about available operating modes consult 23K640 datasheet:
     http://ww1.microchip.com/downloads/en/DeviceDoc/22126C.pdf
 * Test configuration:
     MCU:             dsPIC24FJ96GA010
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39747e.pdf
     dev.board:       LV24-33 v6 - ac:Serial_RAM
                      http://www.mikroe.com/eng/products/view/430/lv-24-33-v6-development-system
     Oscillator:      XT, 8.00000 MHz
     Ext. Modules:    SerialRAM board
                      http://www.mikroe.com/en/tools/serial-ram/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/en/compilers/mikroc/pro/pic/
 * NOTES:
     - Turn on SPI switches SW14.1, SW14.3, SW14.5 and Serial RAM CS and HOLD switches
       SW14.8 and SW15.1
     - Turn on PORTD LEDs at SW13.4 (board specific)
*/

// SerialRAM connections
sbit Chip_Select at LATB0_bit;
sbit HOLD at LATB1_bit;
sbit Chip_Select_Direction at TRISB0_bit;
sbit HOLD_Direction at TRISB1_bit;
// End SerialRAM connections

unsigned int i;
unsigned short tmp;

// Write one byte of data to specified address
void Write_Data(unsigned int address, unsigned short dat){
  Chip_Select = 0;                       // Select SerialRAM
  SPI2_Write(2);                         // Write instruction
  SPI2_Write(address >> 8);              // Sending 16 bits address
  SPI2_Write(address);
  SPI2_Write(dat);                       // Writing one byte of data
  Delay_us(2);
  Chip_Select = 1;                       // Deselect SerialRAM
 }

// Read one byte of data from specified address
unsigned short Read_Data(unsigned int address){
  Chip_Select = 0;                       // Select Serial RAM
  SPI2_Write(3);                         // Read instruction
  SPI2_Write(address >> 8);              // Sending 16 bits address
  SPI2_Write(address);
  tmp = SPI2_Read(0);                    // Read one byte of data
  Delay_us(2);
  Chip_Select = 1;                       // Deselect SerialRAM
  return tmp;
}

void main() {
  ADPCFG = 0xFFFF;                       // configure AN pins as digital

  TRISD  = 0;                            // PORTD is output
  PORTD  = 0;

  Chip_Select_Direction = 0;             // Set CS# pin as Output
  Chip_Select = 1;                       // Deselect SerialRAM
  SPI2_Init();                           // Initialize SPI module
  HOLD_Direction = 0;                    // Set HOLD# pin as output
  HOLD = 1;                              // Hold high when this function is not used

  for (i = 0; i < 256 ; i++){            // Write 256 bytes of data starting from address 0x0A00
    Write_Data(i+0x0A00, i);             // Write data to specified address
    LATD = Read_Data(i+0x0A00);          // Read data and display it on PORTD
    delay_ms(100);
  }
}