/*
 * Project name:
     TouchPanelCalibration (Demo for working with TouchPanel Controller)
 * Copyright:
     (c) Mikroelektronika, 2010.
 * Revision History:
     20100429:
       - initial release;
 * Description:
     This code works with TouchPanel and GLCD. Two digital output and
     two analog input signals are used for communication with TouchPanel.
     The example shows how to calibrate touch panel and prints aquired
     calibration constants on GLCD.
 * Test configuration:
     MCU:             PIC24FJ96GA010
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39747e.pdf
     Dev.Board:       LV24-33 v6 - ac:TP
                      http://www.mikroe.com/eng/products/view/430/lv-24-33-v6-development-system
     Oscillator:      XT, 08.00000 MHz
     Ext. Modules:    GLCD 128x64, Touch Panel
                      http://www.mikroe.com/en/tools/components/
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/en/compilers/mikroc/pro/dspic/
 * Notes:
     - Turn on GLCD backlight(SW17.8) and TouchPanel Contoller switches (SW15.5, SW15.6, SW15.7, SW15.8)(board specific).
     - Turn off LEDs connected to ADC pins.
*/

// Glcd module connections
sbit GLCD_D7 at RD7_bit;
sbit GLCD_D6 at RD6_bit;
sbit GLCD_D5 at RD5_bit;
sbit GLCD_D4 at RD4_bit;
sbit GLCD_D3 at RD3_bit;
sbit GLCD_D2 at RD2_bit;
sbit GLCD_D1 at RD1_bit;
sbit GLCD_D0 at RD0_bit;
sbit GLCD_D7_Direction at TRISD7_bit;
sbit GLCD_D6_Direction at TRISD6_bit;
sbit GLCD_D5_Direction at TRISD5_bit;
sbit GLCD_D4_Direction at TRISD4_bit;
sbit GLCD_D3_Direction at TRISD3_bit;
sbit GLCD_D2_Direction at TRISD2_bit;
sbit GLCD_D1_Direction at TRISD1_bit;
sbit GLCD_D0_Direction at TRISD0_bit;

sbit GLCD_CS1 at LATB2_bit;
sbit GLCD_CS2 at LATB3_bit;
sbit GLCD_RS  at LATB4_bit;
sbit GLCD_RW  at LATB5_bit;
sbit GLCD_EN  at LATB6_bit;
sbit GLCD_RST at LATB7_bit;
sbit GLCD_CS1_Direction at TRISB2_bit;
sbit GLCD_CS2_Direction at TRISB3_bit;
sbit GLCD_RS_Direction  at TRISB4_bit;
sbit GLCD_RW_Direction  at TRISB5_bit;
sbit GLCD_EN_Direction  at TRISB6_bit;
sbit GLCD_RST_Direction at TRISB7_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveA at LATD10_bit;
sbit DriveB at LATD11_bit;
sbit DriveA_Direction at TRISD10_bit;
sbit DriveB_Direction at TRISD11_bit;
// End Touch Panel module connections

unsigned int x_min, y_min, x_max, y_max;           // Calibration constants

char x_min_msg[] = "X min:";                       // GLCD text messages
char y_min_msg[] = "Y min:";
char x_max_msg[] = "X max:";
char y_max_msg[] = "Y max:";

char x_min_val[6];                                 // Calibration constants string values
char y_min_val[6];
char x_max_val[6];
char y_max_val[6];

void Initialize() {
  ADPCFG = 0xFCFF;                                 // Set AN8 and AN9 channel pins as analog
  CMCON = 0;                                       // Turn off comparators

  DriveA_Direction = 0;                            // Set DriveA pin as output
  DriveB_Direction = 0;                            // Set DriveB pin as output

  Glcd_Init();                                     // Initialize GLCD
  Glcd_Fill(0);                                    // Clear GLCD

  ADC1_Init();                                     // Initialize ADC
  TP_Init(128, 64, 8, 9);                          // Initialize touch panel
  TP_Set_ADC_Threshold(900);                       // Set touch panel ADC threshold
}

void Calibrate() {

  Glcd_Dot(0,63,1);                                // Draw bottom left dot
  Glcd_Write_Text("TOUCH BOTTOM LEFT",12,3,1);
  TP_Calibrate_Bottom_Left();                      // Calibration of bottom left corner
  Delay_ms(1000);

  Glcd_Dot(0,63,0);                                // Clear bottom left dot
  Glcd_Dot(127,0,1);                               // Draw upper right dot
  Glcd_Write_Text("                 ",12,3,1);
  Glcd_Write_Text("TOUCH UPPER RIGHT",12,4,1);
  TP_Calibrate_Upper_Right();                      // Calibration of upper right corner

  Delay_ms(1000);
}

void main() {

  Initialize();

  Glcd_Write_Text("CALIBRATION",12,3,1);
  Delay_ms(1000);
  Glcd_Fill(0);                                               // Clear GLCD
  Calibrate();

  TP_Get_Calibration_Consts(&x_min, &y_min, &x_max, &y_max);  // Get calibration constants

  WordToStr(x_min, x_min_val);                                // Convert calibration constants into string values
  WordToStr(y_min, y_min_val);
  WordToStr(x_max, x_max_val);
  WordToStr(y_max, y_max_val);

  // Display Calibration constants on GLCD

  Glcd_Fill(0);                                               // Clear GLCD
  Glcd_Write_Text("Calibration constants",1,1,1);
  Glcd_Write_Text(x_min_msg,1,3,1);                           // Write messages on GLCD
  Glcd_Write_Text(y_min_msg,1,4,1);
  Glcd_Write_Text(x_max_msg,1,5,1);
  Glcd_Write_Text(y_max_msg,1,6,1);

  Glcd_Write_Text(x_min_val,50,3,1);                          // Write calibration constants values on GLCD
  Glcd_Write_Text(y_min_val,50,4,1);
  Glcd_Write_Text(x_max_val,50,5,1);
  Glcd_Write_Text(y_max_val,50,6,1);

}